# coding: utf-8
require 'uzuuzu-cms/model/user'
require 'uzuuzu-cms/model/css'
require 'uzuuzu-cms/model/javascript'
require 'uzuuzu-cms/model/layout'
require 'uzuuzu-cms/model/page_body'
require 'uzuuzu-cms/model/page'

module UzuUzu
  class User
    has n, :infos
  end

  class Css
    has n, :infos, :through => Resource
  end

  class Javascript
    has n, :infos, :through => Resource
  end

  class Layout
    has n, :infos
  end

  class Page
    has n, :infos
  end

  class Info
    include DataMapper::Resource

    property :id, Serial
    property :title, String,
             :required => true,
             :default => '',
             :length => 255
    property :step, Integer,
             :default => 1

    belongs_to :top_page, 'UzuUzu::Page', :required => false
    belongs_to :layout

    has n, :csses, 'UzuUzu::Css', :through => Resource
    has n, :javascripts, 'UzuUzu::Javascript', :through => Resource
    def self.uniq
      ret = nil
      UzuUzu.transaction do |tr|
        tr.begin
        ary = self.all
        if ary.empty?
          fixture_load
        end
        ary = self.all

        ret = ary.pop
        ary.each do |record|
          record.destroy
        end
        tr.commit
      end
      ret
    end #self.uniq
    #
    def self.fixture_load
      # fixture load
      ::UzuUzu.logger.debug 'fixture load start'
      Dir.glob('./fixture/**/*.*').each do |path|
        ::UzuUzu.logger.debug "fixture file : #{path}"
        paths = path.split('/')
        model_name = ::File.basename(paths.pop, '.*')
        while !paths.blank? && paths[-1] != 'fixture'
          model_name = "#{paths.pop.camel_case}::#{model_name.camel_case}"
        end
        begin
          ::UzuUzu.logger.debug "fixture model name : #{model_name}"
          model = eval("::#{model_name}")
          ::UzuUzu::DataStore::Fixture.import_file(model, path, true)
        rescue NameError => e
          ::UzuUzu.logger.error(e.to_s)
        end
      end
      ::UzuUzu.logger.debug 'fixture load end'
    end # fixture_load
    
    def initialize?
      self.step >= 2
    end

    def self.initialize?
      info = self.uniq
      info.initialize?
    end

    def breadcrumbs
      []
    end

    def layout_fix
      self.layout
    end
    
    def css_urls
      self.csses.map do |css|
        css.url
      end
    end
    
    def js_urls
      self.javascripts.map do |js|
        js.url
      end
    end
    
    def contents_render(options={}, locals={})
      ''
    end
    
    def page_title
      self.title
    end
    
    def render(options={}, locals={})
      self.layout.render(options, locals)
    end # render
  end #Info

  if auto_upgrade?
    repository do
      Info.auto_upgrade!
      InfoJavascript.auto_upgrade!
      CssInfo.auto_upgrade!
    end
  end
end #UzuUzu
