# coding: utf-8
require 'uzuuzu-cms/model/css'
require 'uzuuzu-cms/model/javascript'
require 'uzuuzu-cms/model/layout'
require 'uzuuzu-cms/model/contents'
require 'uzuuzu-cms/model/page_meta'

module UzuUzu
  class Css
    has n, :pages, :through => Resource
  end
  
  class Javascript
    has n, :pages, :through => Resource
  end
  
  class Layout
    has n, :pages
  end
  
  class ContentsPage
    include DataMapper::Resource
    
    belongs_to :page, :key => true
    belongs_to :contents, 'UzuUzu::Contents', :key => true
  end
  
  class Contents
    has n, :pages, :through => Resource, :via => :page
  end
  
  class Page
    include DataMapper::Resource
    include DataMapper::Timestamps

    property :id, Serial
    property :name, String,
              :required => true,
              :length => 255
    property :title, String,
              :required => true,
              :length => 255
    timestamps :at

    belongs_to :layout,
                :required => false
    belongs_to :parent, 'UzuUzu::Page',
                :required => false
    
    has n, :csses, :through => Resource
    has n, :javascripts, :through => Resource
    has n, :contents, 'UzuUzu::Contents', :through => Resource, :via => :contents            
    has n, :page_metas, :through => Resource
    has n, :childs, 'UzuUzu::Page', :child_key => :parent_id
    
    def reverse_look_up(column=nil)
      if column
        key = self.send(column)
      else
        key = self
      end
      child_tree = parent.reverse_look_up(self) if parent
      {key => child_tree}
    end
    
    def breadcrumbs
      _breadcrumbs = []
      tree = reverse_look_up
      while(tree)
        tree.each do |key, value|
          _breadcrumbs.push({:id => key.id, :name => key.name})
          tree = value
        end
      end
      _breadcrumbs.reverse
    end
    
    #
    #
    #
    def self.page_id_tree(parent_page_id=nil)
      self.tree(:id, parent_page_id)
    end
    
    #
    #
    #
    def self.page_name_tree(parent_page_id=nil)
      self.tree(:name, parent_page_id)
    end
    
    #
    #
    #
    def self.tree(column=nil, parent_page_id=nil)
      ary = self.all(:parent_page_id => parent_page_id)
      return nil if ary.nil? or ary.length <= 0
      tree_map = {}
      ary.each do |record|
        child_tree = self.tree(column, record.id)
        if colomn
          key = record.send(column)
        else
          key = self
        end
        tree_map[key] = self.tree(column, record.id)
      end
      tree_map
    end
    
    def layout_fix
      value = self.layout
      value ||= self.parent.abs_layout if self.parent
      value ||= ::UzuUzu.info.layout
      value
    end
    
    def css_urls
      value = []
      self.csses.each do |css|
        value << css
      end
      value += self.parent.abs_csses if self.parent
      value += self.layout_fix.csses if self.layout_fix
      value += ::UzuUzu.info.csses unless self.parent
      self.contents.each do |content|
        value += content.csses
      end
      value.map do |css|
        css.url
      end
    end
    
    def js_urls
      value = []
      self.javascripts.each do |js|
        value << js
      end
      value += self.parent.abs_javascripts if self.parent
      value += self.layout_fix.javascripts if self.layout_fix
      value += ::UzuUzu.info.javascripts unless self.parent
      self.contents.each do |content|
        value += content.javascripts
      end
      value.map do |js|
        js.url
      end
    end
    
    def contents_render(options={}, locals={})
      unless @contents_body
        @contents_body = contents.map do |content|
          content.render(options, locals)
        end.join("\n")
      end
      @contents_body
    end
    
    def page_title
      "#{self.title} - #{::UzuUzu.info.title}"
    end
    
    def render(options={}, locals={})
      self.layout_fix.render(options, locals)
    end # render
  end #Page
  
  if auto_upgrade?
    repository do
      Page.auto_upgrade!
      JavascriptPage.auto_upgrade!
      CssPage.auto_upgrade!
      ContentsPage.auto_upgrade!
      PageMeta.auto_upgrade!
      PagePageMeta.auto_upgrade!
    end
  end
end #UzuUzu
